;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.1 - GPL3 - Feb 2012
;
;-----------------------------------------------------------------------------------
; Format command
;-----------------------------------------------------------------------------------

align 16

fs_format:
	xor eax, eax
	mov al, [cli_args]		; Check the argument number		
	cmp al, 1
	je .no_arg
	cmp al, 2
	je .dev
	jmp .no_arg
.dev:
	mov al, 1			; device
	mov ah, 0
	mov rsi, cli_command_args_string
	call os_get_strv

	cmp dword [rsi], 'ata0'
	je .ata0
	cmp dword [rsi], 'ata1'
	je .ata1
	cmp dword [rsi], 'ata2'
	je .ata2
	cmp dword [rsi], 'ata3'
	je .ata3

	jmp .error			; type error

.ata0:
	mov rax, [ata0.tLBA]
	jmp .selected
.ata1:
	mov rax, [ata1.tLBA]
	jmp .selected
.ata2:
	mov rax, [ata2.tLBA]
	jmp .selected
.ata3:
	mov rax, [ata3.tLBA]
	;jmp .selected

.selected:

	mov [tlba_temp], rax		; store values
	mov eax, dword[rsi]

	call current_device		; select
	jc .error

 ;----------------------------------------------------;
 ; Fill the buffer of sector with 0                   ;
 ;----------------------------------------------------;
	mov ecx, 512  
	mov rdi, sector2
.fill:
	mov byte [rdi], 0
	inc rdi
	dec ecx				     
	cmp ecx, 0
	jne .fill

;-----------------------------------------------------------------------;
; Fill the buffer of sector with GUID Partition Entry Format parameters ;
;-----------------------------------------------------------------------;

					; Partition entries (LBA 2)
	; System			
	mov rdi, sector2

	call os_get_uuid_p
	mov qword [rdi+0], rbx		; 16 bytes - Partition type GUID
	mov qword [rdi+8], rcx	 	;

	call os_get_uuid
	mov qword [rdi+16], rbx		; 16 bytes - Unique partition GUID
	mov qword [rdi+24], rcx		;

	mov qword [rdi+32], 40			; 8 bytes - First LBA (little-endian)			***
	mov qword [rdi+40], LBA_SYSTEM_END	; 8 bytes - Last LBA (inclusive, usually odd)		*** 8 MiB

	mov qword [rdi+48], 1		; 8 bytes - Attribute flags ( set bit 0 = System partition, bit 1 = User partition
					;				  bit 60 = Read-only, bit 62 = Hidden )

	; From +56 to +127 : Reserved	; 72 bytes - Partition name (36 UTF-16LE code units)
	;-------------------------------

	; User			

	call os_get_uuid_p
	mov qword [rdi+128], rbx 	; 16 bytes - Partition type GUID
	mov qword [rdi+136], rcx 	;

	call os_get_uuid
	mov qword [rdi+144], rbx	; 16 bytes - Unique partition GUID
	mov qword [rdi+152], rcx	;

	sub rax, 41				; [ t.LBA-41 ]
	mov qword [rdi+160], LBA_SYSTEM_END + 1	; 8 bytes - First LBA (little-endian)			***
	mov qword [rdi+168], rax		; 8 bytes - Last LBA (inclusive, usually odd)		***
	
	mov qword [rdi+176], 2		; 8 bytes - Attribute flags ( set bit 0 = System partition, bit 1 = User partition
					;				  bit 60 = Read-only, bit 62 = Hidden )

	; From +184 to +255 : Reserved	; 72 bytes - Partition name (36 UTF-16LE code units)
	;-------------------------------

	; 3' and 4' partition
	; From +256 to +511 : Reserved	; 256 bytes - Room for two partitions, leaved blank

	
	add rax, 8			; now = t.LBA-33
	push rax			; store value

	; write primary gpt
	;------------------
	mov rax, 2
	mov rbx, [current.base]
	mov rdx, [current.id]
	mov rcx, 1
	mov rsi, sector2
	call [current.writesectors]

	cmp rcx, 0
	je .error
	;----------------------

	; write secondary gpt
	;--------------------
	pop rax 			; t.LBA-33
	mov rbx, [current.base]
	mov rdx, [current.id]
	mov rcx, 1
	mov rsi, sector2
	call [current.writesectors]

	cmp rcx, 0
	je .error
	;----------------------


; ------ GPT Entry End ----------------------------------------------------;


	mov rax, [tlba_temp]		; restore t.LBA value

 ;----------------------------------------------------;
 ; Fill the buffer of sector with 0                   ;
 ;----------------------------------------------------;
	mov ecx, 512  
	mov rdi, sector1
.fill2:
	mov byte [rdi], 0
	inc rdi
	dec ecx				     
	cmp ecx, 0
	jne .fill2

;-----------------------------------------------------------------------;
; Fill the buffer of sector with GUID Partition Table Format parameters ;
;-----------------------------------------------------------------------;
;
; Note: The CRCs are not calculated!

					; Partition table header (LBA 1)
	mov rdi, sector1

	mov byte [rdi+0], 0x45 		; 8 bytes - Signature ("EFI PART", 45 46 49 20 50 41 52 54 hex)
	mov byte [rdi+1], 0x46
	mov byte [rdi+2], 0x49
	mov byte [rdi+3], 0x20
	mov byte [rdi+4], 0x50
	mov byte [rdi+5], 0x41
	mov byte [rdi+6], 0x52
	mov byte [rdi+7], 0x54

	mov byte [rdi+8], 0 		; 4 bytes - Revision (For version 1.0, the value is 00 00 01 00)
	mov byte [rdi+9], 0
	mov byte [rdi+10], 0x01
	mov byte [rdi+11], 0

	mov byte [rdi+12], 0x5C 	; 4 bytes - Header size (in bytes, usually 5C 00 00 00 meaning 92 bytes)
	mov byte [rdi+13], 0
	mov byte [rdi+14], 0
	mov byte [rdi+15], 0

	mov dword [rdi+16], 0		; 4 bytes - CRC32 of header (0 to header size), with this field zeroed during calculation
	mov dword [rdi+20], 0		; 4 bytes - Reserved, must be zero

	mov qword [rdi+24], 1		; 8 bytes - Current LBA (location of this header copy) [ the second sector of disk ]

	dec rax
	mov qword [rdi+32], rax		; 8 bytes - Backup LBA (location of the other header copy) [ the last sector of disk = t.LBA-1]

	mov qword [rdi+40], 34		; 8 bytes - First usable LBA for partitions (primary partition table last LBA + 1)
	sub rax, 33
	mov qword [rdi+48], rax		; 8 bytes - Last usable LBA (secondary partition table first LBA - 1) [ t.LBA-34 ]

	call os_get_uuid
	mov qword [rdi+56], rbx		; 16 bytes - Disk GUID (also referred as UUID on UNIXes)
	mov qword [rdi+64], rcx

	mov qword [rdi+72], 2		; 8 bytes - Partition entries starting LBA (always 2)

	mov dword [rdi+80], 2		; 4 bytes - Number of partition entries
					; 	    (system and user)

	mov dword [rdi+84], 128		; 4 bytes - Size of a partition entry (usually 128)
	mov dword [rdi+88], 0		; 4 bytes - CRC32 of partition array

	; From +92 to End : Reserved	; Must be zeroes for the rest of the block (420 bytes for a 512-byte LBA)
	;-------------------------------


	add rax, 33			; now = t.LBA-1
	push rax			; store value

	; write primary gpt
	;------------------
	mov rax, 1
	mov rbx, [current.base]
	mov rdx, [current.id]
	mov rcx, 1
	mov rsi, sector1
	call [current.writesectors]

	cmp rcx, 0
	je .error
	;----------------------


	; write secondary gpt
	;--------------------
	pop rax 			; t.LBA-1
	mov rbx, [ata0.base]
	mov rdx, [ata0.id]
	mov rcx, 1
	mov rsi, sector1
	call [ata0.writesectors]

	cmp rcx, 0
	je .error
	;----------------------


; ------ GPT Header End ---------------------------------------------------;


 ;----------------------------------------------------;
 ; Fill the buffer of cluster with 0                  ;
 ;----------------------------------------------------;
	mov ecx, 4096  
	mov rdi, buffer64
.fill3:
	mov byte [rdi], 0
	inc rdi
	dec ecx				     
	cmp ecx, 0
	jne .fill3

 ;-----------------------------------------------------;
 ; Fill the buffer of cluster with fs parameters       ;
 ;-----------------------------------------------------;
	mov rdi, buffer64

	mov rax,  'BNFS0001'
	mov qword [rdi+  0], rax		; Signature
	mov rax, [tlba_temp]			; restore t.LBA value
	sub rax, LBA_SYSTEM_END + 1 + 40	; sectors reserved
	shr rax, 3				; div 8
	mov qword [rdi+  8], rax		; File System Size (in clusters)
	sub rax, 4077				; room used by header1, root_p, root, header2, 8 cl. of pointers,
						; init_ffcc dummy cluster and 8 directories (8*508)
	mov qword [rdi+ 16], rax		; Free Space Left (in clusters)
	mov qword [rdi+ 24], 0			; Total number of files
	mov qword [rdi+ 32], 8			; Total number of directories
	mov qword [rdi+ 40], LBA_ROOT_P		; Cluster of Pointers to Root (lba)
	mov qword [rdi+ 48], LBA_INIT_FFCC	; First free clusters chain abs address
	mov qword [rdi+ 56], LBA_INIT_FFCC	; Last free clusters chain abs address
	mov qword [rdi+ 64], rax		; First use counter clusters chain
	mov qword [rdi+ 72], LBA_INIT_FREE	; First free cluster for first use

 ;----------------------------------------------------;
 ; Write hdd : fs header 1 and 2                      ;
 ;----------------------------------------------------;

	; write header 1
	;---------------
	mov rax, LBA_HEADER1
	mov rsi, buffer64
	call write_cluster


	cmp rcx, 0
	je .error
	;----------------------

	; write header 2
	;---------------
	mov rax, LBA_HEADER2
	mov rsi, buffer64
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------


 ;----------------------------------------------------;
 ; Fill the buffer of cluster with 0                  ;
 ;----------------------------------------------------;
	mov ecx, 4096  
	mov rdi, buffer64 + 4096
.fill4:
	mov byte [rdi], 0
	inc rdi
	dec ecx				     
	cmp ecx, 0
	jne .fill4

 ; root
 ;---------------------------------------

	call get_cmos_data
	mov rcx, [date_time]
	mov rdi, root_struc
	add rdi, RECORD_SIZE + 80	; point to archives - datetime
	mov qword[rdi], rcx
	add rdi, RECORD_SIZE		; point to data - datetime
	mov qword[rdi], rcx
	add rdi, RECORD_SIZE		; point to documents - datetime
	mov qword[rdi], rcx
	add rdi, RECORD_SIZE		; point to images - datetime
	mov qword[rdi], rcx
	add rdi, RECORD_SIZE		; point to multimedia - datetime
	mov qword[rdi], rcx
	add rdi, RECORD_SIZE		; point to other - datetime
	mov qword[rdi], rcx
	add rdi, RECORD_SIZE		; point to programs - datetime
	mov qword[rdi], rcx
	add rdi, RECORD_SIZE		; point to sources - datetime
	mov qword[rdi], rcx

	mov rsi, root_struc
	mov rdi, buffer64 + 4096
	mov rcx, [root_struc_size]  
	
.root_str:
	mov al, byte [rsi]
	mov byte [rdi], al
	inc rsi
	inc rdi
	dec rcx				     
	cmp rcx, 0
	jne .root_str

 ;----------------------------------------------------;
 ; Write hdd : root                         (level 0) ;
 ;----------------------------------------------------;

	; write root
	;-----------
	mov rax, LBA_ROOT
	mov rsi, buffer64 + 4096
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------


 ;----------------------------------------------------;
 ; Fill the buffer of cluster	                      ;
 ;----------------------------------------------------;
	mov rdi, buffer64 + 8192
	mov qword[rdi  ], 0
	mov qword[rdi+8], 0
	mov qword[rdi+4080], 0
	mov qword[rdi+4088], CC_PTRS_SIZE	; 16 = offset to first pointer


 ;----------------------------------------------------;
 ; Write hdd : pointers                     (level 1) ;
 ;----------------------------------------------------;

 ; root_p
	mov rdi, buffer64 + 8192
	mov qword[rdi+16], LBA_ROOT
	mov qword[rdi+24], 0
	;write
	mov rax, LBA_ROOT_P
	mov rsi, buffer64 + 8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------
	;----------------------

 ; archives_p
	mov rdi, buffer64 + 8192
	mov rbx, LBA_ARCHIVES
	call fs_set_dir_p
	;write
	mov rax, LBA_ARCHIVES_P
	mov rsi, buffer64 + 8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; data_p
	mov rdi, buffer64 + 8192
	mov rbx, LBA_DATA
	call fs_set_dir_p
	;write
	mov rax, LBA_DATA_P
	mov rsi, buffer64 + 8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; documents_p
	mov rdi, buffer64 + 8192
	mov rbx, LBA_DOCUMENTS
	call fs_set_dir_p
	;write
	mov rax, LBA_DOCUMENTS_P
	mov rsi, buffer64 + 8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; images_p
	mov rdi, buffer64 + 8192
	mov rbx, LBA_IMAGES
	call fs_set_dir_p
	;write
	mov rax, LBA_IMAGES_P
	mov rsi, buffer64 + 8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; multimedia_p
	mov rdi, buffer64 + 8192
	mov rbx, LBA_MULTIMEDIA
	call fs_set_dir_p
	;write
	mov rax, LBA_MULTIMEDIA_P
	mov rsi, buffer64 + 8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; other_p
	mov rdi, buffer64 + 8192
	mov rbx, LBA_OTHER
	call fs_set_dir_p
	;write
	mov rax, LBA_OTHER_P
	mov rsi, buffer64 + 8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; programs_p
	mov rdi, buffer64 + 8192
	mov rbx, LBA_PROGRAMS
	call fs_set_dir_p
	;write
	mov rax, LBA_PROGRAMS_P
	mov rsi, buffer64 + 8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; sources_p
	mov rdi, buffer64 + 8192
	mov rbx, LBA_SOURCES
	call fs_set_dir_p
	;write
	mov rax, LBA_SOURCES_P
	mov rsi, buffer64 +  8192
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------


 ;----------------------------------------------------;
 ; Fill the buffer of cluster with 0                  ;
 ;----------------------------------------------------;
	mov ecx, 4096  
	mov rdi, buffer64 + 12288
.fill6:
	mov byte [rdi], 0
	inc rdi
	dec ecx				     
	cmp ecx, 0
	jne .fill6

 ; directories
 ;---------------------------------------
	mov rdi, buffer64 + 12288

	mov qword[rdi+ 0], 0		; num. of dirs
	mov qword[rdi+ 8], RECORD_SIZE	; fixed lenght
	mov qword[rdi+16], 0		; not used
	mov qword[rdi+24], 0		; not used
	mov qword[rdi+32], 1		; level
	mov rax, 'END0-DIR'
	mov qword[rdi+120], rax		; end rec.0

 ;----------------------------------------------------;
 ; Write hdd : directories                  (level 1) ;
 ;----------------------------------------------------;

 ; archives
	;write
	mov rax, LBA_ARCHIVES
	mov rsi, buffer64 + 12288
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; data
	;write
	mov rax, LBA_DATA
	mov rsi, buffer64 + 12288
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; documents
	;write
	mov rax, LBA_DOCUMENTS
	mov rsi, buffer64 + 12288
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; images
	;write
	mov rax, LBA_IMAGES
	mov rsi, buffer64 + 12288
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; multimedia
	;write
	mov rax, LBA_MULTIMEDIA
	mov rsi, buffer64 + 12288
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; other
	;write
	mov rax, LBA_OTHER
	mov rsi, buffer64 + 12288
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; programs
	;write
	mov rax, LBA_PROGRAMS
	mov rsi, buffer64 + 12288
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

 ; sources
	;write
	mov rax, LBA_SOURCES
	mov rsi, buffer64 + 12288
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------


 ;----------------------------------------------------;
 ; Fill the buffer of cluster with 0                  ;
 ;----------------------------------------------------;
	mov ecx, 4096  
	mov rdi, buffer64 + 16384
.fill7:
	mov byte [rdi], 0
	inc rdi
	dec ecx				     
	cmp ecx, 0
	jne .fill7

	mov rdi, buffer64 + 16384
	mov qword [rdi + 4088], CC_PTRS_SIZE	; offset=16
 ;----------------------------------------------------;
 ; Write hdd : init_ffcc                              ;
 ;----------------------------------------------------;

	;write
	mov rax, LBA_INIT_FFCC
	mov rsi, buffer64 + 16384
	call write_cluster

	cmp rcx, 0
	je .error
	;----------------------

;-----------------------------------------------------

	call fs_probe			; re-init

	jmp .OK

;-----------------------------------------------------
	

.no_arg:
	mov rsi, msg_no_args
	call os_print_string
	call os_print_newline
	jmp .end
.error:
	mov rsi, msg_error
	call os_print_string
	call os_print_newline
	jmp .end
.OK:
	mov rsi, msg_end_command
	call os_print_string
	call os_print_newline
.end:
	ret

;------------------------------

;----------------------------------------
; rdi = buffer
; rbx = start pointer
fs_set_dir_p:
	mov ecx, 508  
	add rdi, CC_PTRS_SIZE
.fill5:
	mov qword [rdi], rbx
	add rdi, 8		; pointer size
	add rbx, 8		; cluster lba
	dec ecx				     
	cmp ecx, 0
	jne .fill5
ret
;----------------------------------------


;----------------------------------------
; fs_descriptor
; IN: 	eax = device
;	ebx = fs present/no present (1/0)
; OUT:	none
;----------------------------------------
fs_descriptor:
	push rax
	push rbx

	cmp ebx, 0
	je .nofs
 ;--------------------
 ; set descriptors on
 ;--------------------
	cmp eax, 'ata0'
	je .ata0d1
	cmp eax, 'ata1'
	je .ata1d1
	cmp eax, 'ata2'
	je .ata2d1
	cmp eax, 'ata3'
	je .ata3d1
	jmp .end_desc

.ata0d1:
	mov al, 1
	mov [ata0.fs], al
	mov rax, ata0_header_cache
	mov [ata0.header], rax

	mov rax, ata0_root_cache
	mov [ata0.root], rax
	jmp .end_desc
.ata1d1:
	mov al, 1
	mov [ata1.fs], al
	mov rax, ata1_header_cache
	mov [ata1.header], rax
	mov rax, ata1_root_cache
	mov [ata1.root], rax

	jmp .end_desc
.ata2d1:
	mov al, 1
	mov [ata2.fs], al
	mov rax, ata2_header_cache
	mov [ata2.header], rax
	mov rax, ata2_root_cache
	mov [ata2.root], rax
	jmp .end_desc
.ata3d1:
	mov al, 1
	mov [ata3.fs], al
	mov rax, ata3_header_cache
	mov [ata3.header], rax
	mov rax, ata3_root_cache
	mov [ata3.root], rax
	jmp .end_desc


 ;--------------------
 ; set descriptors off
 ;--------------------
.nofs:
	cmp eax, 'ata0'
	je .ata0d0
	cmp eax, 'ata1'
	je .ata1d0
	cmp eax, 'ata2'
	je .ata2d0
	cmp eax, 'ata3'
	je .ata3d0
	jmp .end_desc

.ata0d0:
	mov al, 0
	mov [ata0.fs], al
	mov rax, 0
	mov [ata0.header], rax
	mov rax, 0
	mov [ata0.root], rax
	jmp .end_desc
.ata1d0:
	mov al, 0
	mov [ata1.fs], al
	mov rax, 0
	mov [ata1.header], rax
	mov rax, 0
	mov [ata1.root], rax
	jmp .end_desc
.ata2d0:
	mov al, 0
	mov [ata2.fs], al
	mov rax, 0
	mov [ata2.header], rax
	mov rax, 0
	mov [ata2.root], rax
	jmp .end_desc
.ata3d0:
	mov al, 0
	mov [ata3.fs], al
	mov rax, 0
	mov [ata3.header], rax
	mov rax, 0
	mov [ata3.root], rax
	jmp .end_desc


.end_desc:
	pop rbx
	pop rax
	ret
;---------------------------------


;---------------------------------------
; fs_probe
; IN: 	none
; OUT:	none regs (fs descriptors)
;---------------------------------------
fs_probe:
	push rax
	push rbx
	push rcx
	push rdx
	push rsi
	push rdi

	; check ata0
	;-------------------------
	mov al, byte[ata0.present]
	cmp al, 1
	jne .next0

	mov rax, LBA_HEADER1
	mov rbx, [ata0.base]
	mov rdx, [ata0.id]
	mov rcx, 8
	mov rdi, header_buffer
	call [ata0.readsectors]

	cmp rcx, 0
	je .next0

	mov rsi, header_buffer
	mov eax, dword[rsi]
	mov ebx, 'BNFS'
	cmp eax, ebx
	je .ok0
	jmp .nofs0

.ok0:
	mov eax, 'ata0'
	mov ebx, 1
	call fs_descriptor
	jmp .next0
.nofs0:
	mov eax, 'ata0'
	mov ebx, 0
	call fs_descriptor
.next0:
;------------------------------

	; check ata1
	;-------------------------
	mov al, byte[ata1.present]
	cmp al, 1
	jne .next1

	mov rax, LBA_HEADER1
	mov rbx, [ata1.base]
	mov rdx, [ata1.id]
	mov rcx, 8
	mov rdi, header_buffer
	call [ata1.readsectors]

	cmp rcx, 0
	je .next1

	mov rsi, header_buffer
	mov eax, dword[rsi]
	mov ebx, 'BNFS'
	cmp eax, ebx
	je .ok1
	jmp .nofs1

.ok1:
	mov eax, 'ata1'
	mov ebx, 1
	call fs_descriptor
	jmp .next1
.nofs1:
	mov eax, 'ata1'
	mov ebx, 0
	call fs_descriptor
.next1:
;------------------------------

	; check ata2
	;-------------------------
	mov al, byte[ata2.present]
	cmp al, 1
	jne .next2

	mov rax, LBA_HEADER1
	mov rbx, [ata2.base]
	mov rdx, [ata2.id]
	mov rcx, 8
	mov rdi, header_buffer
	call [ata2.readsectors]

	cmp rcx, 0
	je .next2

	mov rsi, header_buffer
	mov eax, dword[rsi]
	mov ebx, 'BNFS'
	cmp eax, ebx
	je .ok2
	jmp .nofs2

.ok2:
	mov eax, 'ata2'
	mov ebx, 1
	call fs_descriptor
	jmp .next2
.nofs2:
	mov eax, 'ata2'
	mov ebx, 0
	call fs_descriptor
.next2:
;------------------------------

	; check ata3
	;-------------------------
	mov al, byte[ata3.present]
	cmp al, 1
	jne .next3

	mov rax, LBA_HEADER1
	mov rbx, [ata3.base]
	mov rdx, [ata3.id]
	mov rcx, 8
	mov rdi, header_buffer
	call [ata3.readsectors]

	cmp rcx, 0
	je .next3

	mov rsi, header_buffer
	mov eax, dword[rsi]
	mov ebx, 'BNFS'
	cmp eax, ebx
	je .ok3
	jmp .nofs3

.ok3:
	mov eax, 'ata3'
	mov ebx, 1
	call fs_descriptor
	jmp .next3
.nofs3:
	mov eax, 'ata3'
	mov ebx, 0
	call fs_descriptor
.next3:
;------------------------------
	
	; load roots

	mov eax, 'ata0'		; load caches
	call fs_load_level0	; (if device present checked in)
	mov eax, 'ata1'
	call fs_load_level0
	mov eax, 'ata2'
	call fs_load_level0
	mov eax, 'ata3'
	call fs_load_level0

	; load headers

	mov eax, 'ata0'		; load caches
	call fs_load_header	; (if device present checked in)
	mov eax, 'ata1'
	call fs_load_header
	mov eax, 'ata2'
	call fs_load_header
	mov eax, 'ata3'
	call fs_load_header
;------------------------------



.end:
	pop rdi
	pop rsi
	pop rdx
	pop rcx
	pop rbx
	pop rax
	ret


; -----------------------------------------------------------------------------
; os_get_uuid -- Return a random UUID
; OUT:  RBX = first part 64 bit
; 	RCX = second part 64 bit
os_get_uuid:
	push rax
	push rdi

	call os_get_random
	mov [uuid1], rax
	call os_get_random
	mov [uuid2], rax

	mov rdi, uuid1			; fixed parts for this OS
	mov word [rdi+6], 0x4444
	mov rdi, uuid2
	mov byte [rdi+0], 0xba

	mov rbx, [uuid1]
	mov rcx, [uuid2]

	pop rdi
	pop rax
	ret

; Version 4 (random)
; Version 4 UUIDs use a scheme relying only on random numbers.
; This algorithm sets the version number as well as two reserved bits.
; All other bits are set using a random or pseudorandom data source.
; Version 4 UUIDs have the form xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx
; where x is any hexadecimal digit and y is one of 8, 9, A, or B.
; e.g. f47ac10b-58cc-4372-a567-0e02b2c3d479.
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; os_get_uuid_p -- Return a partition UUID
; OUT:  RBX = first part 64 bit
; 	RCX = second part 64 bit
os_get_uuid_p:
	push rax
	push rdi

	mov rdi, uuid1
	mov rcx, 0
.again1:
	mov word [rdi+rcx], 0xbaba
	add rcx, 2
	cmp rcx, 8
	jne .again1

	mov rdi, uuid2
	mov rcx, 0
.again2:
	mov word [rdi+rcx], 0xbaba
	add rcx, 2
	cmp rcx, 8
	jne .again2

	mov rbx, [uuid1]
	mov rcx, [uuid2]

	pop rdi
	pop rax
	ret
; -----------------------------------------------------------------------------

uuid1 dq 0	; 
uuid2 dq 0	; 


;-----------------------------------------------------------------------------------------------------------------------
; GPT is little endian: least significant byte first.
; The protective MBR is the entire area of the disk used by GPT structures and partitions.
; In GPT, GUIDs are considered to have a structure:
; the first three dash-delimited fields of the GUID are stored little endian, but the last two fields are not.
; e.g.: 12345678-abcd-ef09-b12b-0a0b0c0d9a9b9c9d is written to disk
;       0x78 0x56 0x34 0x12   0xcd 0xab   0x09 0xef  0xb1 0x2b   0x0a 0x0b 0x0c 0x0d 0x9a 0x9b 0x9c 0x9d
;
; The partition entry array is at least 16 KiB long. (512 byte block size -> 32 blocks, 4096 byte block size -> 4 blocks)
; Do not assume that the block size is always 512 bytes long.
; ----------------------------------------------------------------------------------------------------------------------

;------------------------------------------------------------
; GUID Partition Table Scheme
;
; -----------------------------------------------------------
; LBA 0		MBR
;------------------------------------------------------------
; LBA 1		Primary GPT Header
;------------------------------------------------------------
; LBA 2		Entry 1 - Entry 2 - Entry 3 - Entry 4
;------------------------------------------------------------
; LBA 3
; ...		Entry 5 - 128
; ...
; ...
;------------------------------------------------------------
; LBA 34	Partition 1
;
; ...		Partition 2
;
;		...
;		...
;
;------------------------------------------------------------
; LBA-33	Entry 1 - Entry 2 - Entry 3 - Entry 4
;------------------------------------------------------------
; ...
; ...
; ...		Entry 5 - 128
; LBA-2
;------------------------------------------------------------
; LBA-1		Secondary GPT Header
;------------------------------------------------------------
;
;
; GUID Partition Table Scheme used by me
;
; -----------------------------------------------------------
; LBA 0		MBR (Not written on HDD with fs_format)
;------------------------------------------------------------
; LBA 1		Primary GPT Header
;------------------------------------------------------------
; LBA 2		Entry 1 - Entry 2 - Entry 3 - Entry 4
;------------------------------------------------------------
; LBA 3
; ...		Entry 5 - 128
; ...
; ...
;------------------------------------------------------------
; LBA 34-39	Not used
;------------------------------------------------------------
; LBA 40	Partition 1 - System
; ...
; LBA 16423	16384 sectors = 8 MiB
;------------------------------------------------------------
; LBA 16424	Partition 2 - User
; ...
; ...
; ...
; LBA-41
;------------------------------------------------------------
; LBA-40 -> -34	Not used
;------------------------------------------------------------
; LBA-33	Entry 1 - Entry 2 - Entry 3 - Entry 4
;------------------------------------------------------------
; ...
; ...
; ...		Entry 5 - 128
; LBA-2
;------------------------------------------------------------
; LBA-1		Secondary GPT Header
;------------------------------------------------------------




;-----------------------------------------------------------------
; For more information about file system structure view fs.inc too 
;-----------------------------------------------------------------
