;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.1 - GPL3 - Feb 2012
;
;-----------------------------------------------------------------------------------
; IDT
;-----------------------------------------------------------------------------------


IDTR:					; Interrupt Descriptor Table Register
  dw 256*16-1				; limit of IDT (size minus one)
  dq 0					; linear address of IDT


K_Init_IDT:

	xor rdi, rdi 			; create the 64-bit IDT (at linear address 0x0000000000000000)

	; Create exception gate stubs (Sys64 has already set the correct gate markers)
	;----------------------------
	mov rcx, 32
	mov rax, exception_gate
make_exception_gate_stubs:
	call create_gate
	add rdi, 1
	sub rcx, 1
	jnz make_exception_gate_stubs

	; Create interrupt gate stubs (Sys64 has already set the correct gate markers)
	;----------------------------
	mov rcx, 256-32
	mov rax, interrupt_gate
make_interrupt_gate_stubs:
	call create_gate
	add rdi, 1
	sub rcx, 1
	jnz make_interrupt_gate_stubs

	; Set up the exception gates for all of the CPU exceptions
	;---------------------------------------------------------
	mov rcx, 20
	xor rdi, rdi
	mov rax, exception_gate_00
make_exception_gates:
	call create_gate
	add rdi, 1
	add rax, 16			; The exception gates are aligned at 16 bytes
	sub rcx, 1
	jnz make_exception_gates


	; [0x20] IRQ 0 handler	(System Timer)
	; [0x21] IRQ 1 handler	(Keyboard)
	; [0x22] IRQ 2 handler	(Cascade Controller 2nd PIC from IRQ9)
	; [0x23] IRQ 3 handler	(COM 2/4)
	; [0x24] IRQ 4 handler	(COM 1/3)
	; [0x25] IRQ 5 handler	(*Sound card, Parallel Port 2, Modem)
	; [0x26] IRQ 6 handler	(Floppy)
	; [0x27] IRQ 7 handler	(Parallel Port 1)
	; [0x28] IRQ 8 handler	(Real-Time Clock)
	; [0x29] IRQ 9 handler	(*Redirected IRQ 2, Network)
	; [0x2A] IRQ 10 handler	(*Sound card, USB, SCSI)
	; [0x2B] IRQ 11 handler	(*Network, SCSI, Video)
	; [0x2C] IRQ 12 handler	(PS/2 Mouse)
	; [0x2D] IRQ 13 handler	(Coprocessor)
	; [0x2E] IRQ 14 handler	(Primary IDE Hard Drive)
	; [0x2F] IRQ 15 handler	(Secondary IDE Hard Drive)


	; Set up the IRQ handlers
	;------------------------
	mov rdi, 0x20
	mov rax, clock
	call create_gate

	mov rdi, 0x21
	mov rax, keyboard
	call create_gate

	mov rdi, 0x22
	mov rax, cascade
	call create_gate

	mov rdi, 0x28
	mov rax, rtc
	call create_gate

	mov rdi, 0x80
	mov rax, ap_wakeup
	call create_gate

	mov rdi, 0x81
	mov rax, ap_reset
	call create_gate

	lidt	[IDTR]			; load IDT register

ret



; create_gate
; IN : rax = address of handler
;      rdi = gate # to configure
;--------------------------------
create_gate:
	push rdi
	push rax
	
	shl rdi, 4			; quickly multiply rdi by 16
	stosw				; store the low word (15..0)
	shr rax, 16
	add rdi, 4			; skip the gate marker
	stosw				; store the high word (31..16)
	shr rax, 16
	stosd				; store the high dword (63..32)

	pop rax
	pop rdi
ret


; ===================================================================================================
; EOF


;                                80386 INTERRUPT GATE
;   31                23                15                7                0
;  +-----------------+-----------------+---+---+-+-------+-----+-----------+
;  |           OFFSET 31..16           | P |DPL|S|1 1 1 0|0 0 0|(NOT USED) |4
;  |-----------------------------------+---+---+-+-------+-----+-----------|
;  |             SELECTOR              |           OFFSET 15..0            |0
;  +-----------------+-----------------+-----------------+-----------------+


; IDT Descriptor:    (64bit)
; Offset Size 	Description
; 0 	 2 	Offset low bits (0..15)
; 2 	 2 	Selector (Code segment selector)
; 4 	 1 	Zero
; 5 	 1 	Type and Attributes
; 6 	 2 	Offset middle bits (16..31)
; 8 	 4 	Offset high bits (32..63)
; 12 	 4 	Zero

; Here are some pre-cooked type_attr values people are likely to use (assuming DPL=0): 
; 32-bit Interrupt gate: 0x8E ( P=1, DPL=00b, S=0, type=1110b => type_attr=1000_1110b=0x8E) 
; 32-bit Trap gate: 0x8F ( P=1, DPL=00b, S=0, type=1111b => type_attr=1000_1111b=0x8F) 

