;-----------------------------------------------------------------------------------
; BareNumbersOS v.0.1 - GPL3 - Feb 2012
;
;-----------------------------------------------------------------------------------
;
; This code is based on:
; - BareMetalOS - Pure64 : http://www.returninfinity.com/baremetal.html
; - OSdev Wiki : http://wiki.osdev.org
; - Code available at FASM site : http://www.flatassembler.net
;-----------------------------------------------------------------------------------


USE64
ORG 0x0000000000400000

kernel_start:

jmp start				; Skip over
	
	nop
	db 'BareNumbersOS'

;====================================================;
;  include 64-bit kernel.                            ;
;====================================================;
	include 'include_k\ksysvar.inc'
	include 'include_k\kmessages.inc'
	include 'include_k\ksyscalls.inc'
	include 'include_k\kinit.inc'
	include 'include_k\kIDT.inc'
	include 'include_k\kinterrupt.inc'
	include 'include_k\kimem_map.inc'
	include 'include_k\kreset_ap.inc'
	include 'include_k\kcli.inc'
	include 'include_k\kcmos.inc'
	include 'include_k\khdd_ata_id.inc'

	include 'include_k\drivers\hdd_ata.inc'
	include 'include_k\drivers\hdd_info.inc'
	include 'include_k\drivers\devices.inc'
	include 'include_k\drivers\fs_structures.inc'
	include 'include_k\drivers\fs.inc'
	include 'include_k\drivers\fs_format.inc'

align 16
start:

	cli				; Disable Interrupt


	; Re-Initialize IDT
	;------------------
	call K_Init_IDT

	; Some initializations
	;---------------------
	call K_Init			; RTC, keyboard, interrupts

	call K_InfoMemory_map

	call K_Reset_ap			; (Use 0x5700 flags)

	call K_HDD_ATA_Identify
		
	call os_seed_random		; Seed the RNG

	call fs_probe			; Check fs and allocate caches


	; Message
	mov ax, 0x0008
	call os_move_cursor
	mov rsi, msg_welcome		
	mov bl, 0x02			; Color
	call os_print_string_with_color

;----------------------------------------


align 16

ap_clear:				; All cores start here on first startup and after an exception

	cli				; Disable interrupts on this core

	; Get local ID of the core
	mov rsi, [os_LocalAPICAddress]
	add rsi, 0x20			; Add the offset for the APIC ID
	lodsd				; Load a 32-bit value. We only want the high 8 bits
	shr rax, 24			; Shift to the right and AL now holds the CPU's APIC ID

	; Calculate offset into CPU status table
	mov rdi, cpustatus
	add rdi, rax			; RDI points to this cores status byte (we will clear it later)

	; Reset the stack
	shl rax, 12			; shift left 12 bits for a 4096 bytes stack
	add rax, 0x0000000000201000	; stacks decrement when you "push", start at 4096 bytes in
	mov rsp, rax			; 

	; Set the CPU status to "Present" and "Ready"
	mov al, 00000001b		; Bit 0 set for "Present", Bit 1 clear for "Ready"
	stosb				; Set status to Ready for this CPU


	sti				; Re-enable interrupts on this core

	; Clear registers. Gives us a clean slate to work with
	xor rax, rax			; aka r0
	xor rcx, rcx			; aka r1
	xor rdx, rdx			; aka r2
	xor rbx, rbx			; aka r3
	xor rbp, rbp			; aka r5, We skip RSP (aka r4) as it was previously set
	xor rsi, rsi			; aka r6
	xor rdi, rdi			; aka r7
	xor r8, r8
	xor r9, r9
	xor r10, r10
	xor r11, r11
	xor r12, r12
	xor r13, r13
	xor r14, r14
	xor r15, r15




ap_spin:				; Spin until there is a workload in the queue
	cmp word [os_QueueLen], 0	; Check the length of the queue
	je ap_halt			; If the queue was empty then jump to the HLT
	call os_smp_dequeue		; Try to pull a workload out of the queue
	jnc ap_process			; Carry clear if successful, jump to ap_process

ap_halt:				; Halt until a wakup call is received
	hlt				; If carry was set we fall through to the HLT
	jmp ap_spin			; Try again

ap_process:				; Set the status byte to "Busy" and run the code
	push rdi			; Push RDI since it is used temporarily
	push rax			; Push RAX since os_smp_get_id uses it
	mov rdi, cpustatus
	call os_smp_get_id		; Set RAX to the APIC ID
	add rdi, rax
	mov al, 00000011b		; Bit 0 set for "Present", Bit 1 set for "Busy"
	stosb
	pop rax				; Pop RAX (holds the workload code address)
	pop rdi				; Pop RDI (holds the variable/variable address)

	call rax			; Run the code

	jmp ap_clear			; Reset the stack, clear the registers, and wait for something else to work on




;--------------------------------------
;	hlt			; debug
;endk:				
;	nop
;	jmp endk


;---------------------------------
;Variable section
;---------------------------------
; Pad to an even KB file (64 KiB)

times 65536-($-$$) db 1


; ==============================================================================================
; EOF



;	This program is free software: you can redistribute it and/or modify it under the terms
;	of the GNU General Public License as published by the Free Software Foundation,
;	either version 3 of the License, or (at your option) any later version.
;       This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
;	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
;	See the GNU General Public License for more details.



